@@ -1,5 +1,5 @@
 /* cdbackup.h.
-Copyright (c) 2000-2002 Craig Condit, Stefan Hlswitt.
+Copyright (c) 2000-2004 Craig Condit, Stefan Hlswitt.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met: 
@@ -34,19 +34,30 @@ SUCH DAMAGE.
 
 #define MIN_BLOCKS	2   /* min. required blocks for a session: header block, one data block */
 
+#define F_NONE   0   /* flag values */
+#define F_CRC    1   /* backup has CRC checksum */
+
 struct header_block {
   char id_str[33];	/* recorder id string (32 chars) */
   char vol_id[33];	/* volume label (32 characters) */
   char t_stamp[13];	/* timestamp */
   char disk_set;	/* disk number - starts with 1 on multi-disk set */
+  char flags;           /* flags for the backup set */
 };
 
 struct data_block {
   char status;		/* status of block (0=continue, 1=done, 2=disk full) */
-  char reserved;	/* reserved byte to make structure 4 bytes long */
+  char flags;		/* flags for this data block */
   short datasize;	/* # of bytes in block (max = CD_FRAMESIZE-this) */
 };
 
+extern int padsize;
+extern long cd_len;
+extern int verbose;
+extern char *prg_name;
+
+void start_cdrecord(void);
+
 #define DBSIZE   sizeof(struct data_block)
 #define DATASIZE (CD_FRAMESIZE-DBSIZE)
 
