@@ -1,5 +1,5 @@
 /* cdrom.h
-Copyright (c) 2000-2002 Craig Condit, Stefan Hlswitt.
+Copyright (c) 2000-2004 Craig Condit, Stefan Hlswitt.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met: 
@@ -26,28 +26,33 @@ SUCH DAMAGE.
 #ifndef _CDBACKUP_CDROM_H
 #define _CDBACKUP_CDROM_H
 
-extern struct toc_entry *toc;
-extern long long cd_used, cd_avail;
+#ifdef linux
+#include <linux/cdrom.h>
+#endif
 
-struct toc_entry {
-  u_char track_no;	/* track number */
-  char is_data;		/* 1 = data track */
-  char is_cdbackup;	/* was it created by CD-Backup? */
-  int sec_start;	/* start sector */
-  int sec_end;		/* last sector */
-  char id_str[33];	/* recorder id string (32 chars) */
-  char vol_id[33];	/* volume label (32 characters) */
-  char t_stamp[13];	/* time stamp: yyyymmddhhmm */
-  char disk_set;	/* disk number */
-};
+#ifdef sun
+#include <sys/cdio.h>
+#endif
+
+#ifndef CD_FRAMESIZE
+#define CD_FRAMESIZE 2048
+#endif
+
+struct cd_header {
+  int start_track, end_track;
+  int used;
+  };
+
+struct cd_track {
+  int start_sec;
+  int leadout_size;
+  int is_data;
+  int start_track;
+  };
 
-int open_cdr(char *device);
-void close_cdr(int cd_fd);
 void get_param(int fd, unsigned long *ahead, unsigned long *fahead);
 void set_param(int fd, unsigned long ahead, unsigned long fahead);
-int full_read(int fd, void *buf, int count);
-int read_toc(int cd_fd, int trackinfos);
-void free_toc();
-void print_space();
+int getCdHeader(struct cd_header *cd);
+void getCdTrack(int num, struct cd_track *cd);
 
 #endif
