@@ -1,5 +1,5 @@
 /* misc.h
-Copyright (c) 2000-2002 Craig Condit, Stefan Hlswitt.
+Copyright (c) 2000-2004 Craig Condit, Stefan Hlswitt.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions are met: 
@@ -26,8 +26,12 @@ SUCH DAMAGE.
 #ifndef _CDBACKUP_MISC_H
 #define _CDBACKUP_MISC_H
 
-int start_diskchange(char *multicmd, char *cd_dev);
+unsigned long crc32(char *data, int len);
+void diskchange(char *multicmd, char *cd_dev);
+int full_read(int fd, void *buf, int count);
 void error(char *text);
 void serror(char *text);
+char *FlexSize(char *buff, long long val);
+long long FlexLen(char *optarg);
 
 #endif
